@@ -24,8 +24,7 @@
 #' in the call.
 #'
 #' @examples
-#' \donttest{
-#' ## Generate data.
+#' \donttest{## Generate data.
 #' set.seed(1986)
 #'
 #' n <- 1000
@@ -68,7 +67,7 @@
 #'
 #' ## To predict, do the following.
 #' tree <- subtree(groupings$tree, cv = TRUE) # Select by cross-validation.
-#' predict(tree, data.frame(X))
+#' head(predict(tree, data.frame(X)))
 #'
 #' ## Inference with 4 groups.
 #' results <- inference_aggtree(groupings, n_groups = 4)
@@ -124,8 +123,12 @@
 #'
 #' \deqn{score_i = \sum_{l = 1}^{|T|} L_{i, l} \beta_l + \epsilon_i}
 #'
-#' This way, betas again identify the GATEs. Regardless of \code{method}, standard errors are estimated via the Eicker-Huber-White
-#' estimator.
+#' This way, betas again identify the GATEs.\cr
+#'
+#' Regardless of \code{method}, standard errors are estimated via the Eicker-Huber-White estimator.\cr
+#'
+#' If \code{boot_ci == TRUE}, the routine also computes asymmetric bias-corrected and accelerated 95% confidence intervals using 2000 bootstrap
+#' samples. Particularly useful when the honest sample is small-ish.
 #'
 #' ### Hypothesis testing
 #' \code{\link{inference_aggtree}} uses the standard errors obtained by fitting the linear models above to test the hypotheses
@@ -162,7 +165,7 @@ build_aggtree <- function(y, D, X,
                           cates = NULL, is_honest = NULL, ...) {
   ## Handling inputs and checks.
   if (any(!(D %in% c(0, 1)))) stop("Invalid 'D'. Only binary treatments are allowed.", call. = FALSE)
-  if (!is.matrix(X) & !is.data.frame(X)) stop("'X' must be either a matrix or a data frame.", call. = FALSE)
+  if (!is.matrix(X) & !is.data.frame(X)) stop("Invalid 'X'. This must be either a matrix or a data frame.", call. = FALSE)
   if (honest_frac < 0 | honest_frac >= 1) stop("Invalid 'honest_frac'. This must be in the interval [0, 1).", call. = FALSE)
   if (!is.null(cates)) if(is.null(is_honest)) stop("When 'cates' is used, you need to specify honest observations via the argument 'is_honest'.", call. = FALSE)
 
@@ -223,18 +226,24 @@ build_aggtree <- function(y, D, X,
 #'
 #' @param object An \code{aggTrees} object.
 #' @param n_groups Number of desired groups.
+#' @param boot_ci Logical, whether to compute bootstrap confidence intervals.
+#' @param boot_R Number of bootstrap replications. Ignored if \code{boot_ci == FALSE}.
 #'
 #' @rdname build_aggtree
 #'
-#' @import car
+#' @importFrom broom tidy
+#' @importFrom boot boot
 #'
 #' @export
-inference_aggtree <- function(object, n_groups) {
+inference_aggtree <- function(object, n_groups,
+                              boot_ci = FALSE, boot_R = 2000) {
   ## Handling inputs and checks.
-  if (!(inherits(object, "aggTrees"))) stop("You must provide a valid aggTrees object.", call. = FALSE)
-  if (!(inherits(object$tree, "rpart"))) stop("You must provide a valid aggTrees object.", call. = FALSE)
+  if (!(inherits(object, "aggTrees"))) stop("Invalid 'object'. This must be an aggTrees object.", call. = FALSE)
+  if (!(inherits(object$tree, "rpart"))) stop("Invalid 'object'. This must be a valid aggTrees object.", call. = FALSE)
   if (is.null(object$idx$honest_idx)) warning("Inference is not valid, because the same data have been used to construct the tree and estimate GATEs.")
   if (n_groups <= 1) stop("Invalid 'n_groups'. This must be greater than or equal to 2.", call. = FALSE)
+  if (!(boot_ci %in% c(FALSE, TRUE))) stop("Invalid 'boot_ci'. This must be either FALSE or TRUE.", call. = FALSE)
+  if (boot_R < 0) stop("Invalid 'boot_R'. This must be a positive integer.", call. = FALSE)
 
   tree <- object$tree
 
@@ -257,11 +266,12 @@ inference_aggtree <- function(object, n_groups) {
   ## Select granularity level.
   groups <- subtree(tree, leaves = n_groups)
 
-  ## GATEs point estimates and standard errors, and hypotheses testing.
-  results <- causal_ols_rpart(groups, y, D, X, method = method, scores = scores)
+  ## GATEs point estimates and standard errors, hypotheses testing, and boostrap CI if required.
+  results <- causal_ols_rpart(groups, y, D, X, method = method, scores = scores, boot_ci = boot_ci, boot_R = boot_R)
 
   model <- results$model
   gates_diff_pairs <- results$gates_diff_pairs
+  boot_ci <- results$boot_ci
 
   ## Compute average characteristics of units in each leaf.
   avg_characteristics <- avg_characteristics_rpart(groups, X)
@@ -270,6 +280,7 @@ inference_aggtree <- function(object, n_groups) {
   output <- list("aggTree" = object,
                  "groups" = groups,
                  "model" = model,
+                 "boot_ci" = boot_ci,
                  "gates_diff_pairs" = gates_diff_pairs,
                  "avg_characteristics" = avg_characteristics)
   class(output) <- "aggTrees.inference"
