@@ -53,7 +53,6 @@
 #' @export
 plot.aggTrees <- function(x, leaves = get_leaves(x$tree), sequence = FALSE, ...) {
   ## Handling inputs and checks.
-  if (!(inherits(x, "aggTrees"))) stop("You must provide a valid aggTrees object.", call. = FALSE)
   if (!(sequence %in% c(TRUE, FALSE))) stop("Invalid 'sequence'. This must be either TRUE or FALSE.", call. = FALSE)
 
   tree <- x$tree
@@ -277,8 +276,14 @@ print.aggTrees.inference <- function(x, table = "avg_char", ...) {
   ## Check.
   if (!(table %in% c("avg_char", "diff"))) stop("Invalid 'table'. This must be either 'avg_char' or 'diff'.", call. = FALSE)
 
+  ## Select appropriate sample (adaptive/honest) according to the output of build_aggtree.
+  if (is.null(x$idx$honest_idx)) {
+    X <- x$aggTree$dta[, -c(1,2 )]
+  } else {
+    X <- x$aggTree$dta[x$aggTree$idx$honest_idx, -c(1,2 )]
+  }
+
   ## Extract information.
-  X <- x$aggTree$dta[x$aggTree$idx$honest_idx, -c(1,2 )]
   leaves <- leaf_membership(x$groups, X)
   parms <- lapply(x$avg_characteristics, function(x) {stats::coef(summary(x))[, c("Estimate", "Std. Error")]})
 
@@ -294,6 +299,9 @@ print.aggTrees.inference <- function(x, table = "avg_char", ...) {
   gates_ci_lower <- round(gates_point - 1.96 * gates_sd, 3)
   gates_ci_upper <- round(gates_point + 1.96 * gates_sd, 3)
 
+  if (length(x$boot_ci) != 0) gates_ci_lower_boot <- round(x$boot_ci$lower, 3) else gates_ci_lower_boot <- NA
+  if (length(x$boot_ci) != 0) gates_ci_upper_boot <- round(x$boot_ci$upper, 3) else gates_ci_upper_boot <- NA
+
   ## Write table.
   if (table == "avg_char") {
     table_names <- rename_latex(colnames(X))
@@ -308,7 +316,7 @@ print.aggTrees.inference <- function(x, table = "avg_char", ...) {
       \\\\[-1.8ex]\\hline
       \\hline \\\\[-1.8ex]
       & ", c(paste("\\multicolumn{2}{c}{\\textit{Leaf ", 1:(length(unique(leaves))-1), "}} & ", sep = ""), paste("\\multicolumn{2}{c}{\\textit{Leaf ", length(unique(leaves)), "}}", sep = "")) ," \\\\",  paste0("\\cmidrule{", seq(2, length(unique(leaves)) * 2, by = 2), "-", seq(3, (length(unique(leaves)) * 2)+1, by = 2), "} "), "
-     ", rep(" & Mean & (S.D.)", length(unique(leaves))), " \\\\
+     ", rep(" & Mean & (S.E.)", length(unique(leaves))), " \\\\
       \\addlinespace[2pt]
       \\hline \\\\[-1.8ex] \n\n", sep = "")
 
@@ -360,8 +368,9 @@ print.aggTrees.inference <- function(x, table = "avg_char", ...) {
       \\addlinespace[2pt]
       \\hline \\\\[-1.8ex] \n\n", sep = "")
 
-    cat("      \\multirow{2}{*}{GATEs} & ", paste(gates_point[1:(length(unique(leaves))-1)], " & ", sep = ""), gates_point[length(unique(leaves))], " \\\\
-      & ", paste("[", gates_ci_lower[1:(length(unique(leaves))-1)], ", ", gates_ci_upper[1:(length(unique(leaves))-1)], "] & ", sep = ""), paste("[", gates_ci_lower[length(unique(leaves))], ", ", gates_ci_upper[length(unique(leaves))], "]", sep = ""), " \\\\ \n\n", sep = "")
+    cat("      \\multirow{3}{*}{GATEs} & ", paste(gates_point[1:(length(unique(leaves))-1)], " & ", sep = ""), gates_point[length(unique(leaves))], " \\\\
+      & ", paste("[", gates_ci_lower[1:(length(unique(leaves))-1)], ", ", gates_ci_upper[1:(length(unique(leaves))-1)], "] & ", sep = ""), paste("[", gates_ci_lower[length(unique(leaves))], ", ", gates_ci_upper[length(unique(leaves))], "]", sep = ""), " \\\\
+      & ", paste("[", gates_ci_lower_boot[1:(length(unique(leaves))-1)], ", ", gates_ci_upper_boot[1:(length(unique(leaves))-1)], "] & ", sep = ""), paste("[", gates_ci_lower_boot[length(unique(leaves))], ", ", gates_ci_upper_boot[length(unique(leaves))], "]", sep = ""), " \\\\ \n\n", sep = "")
     cat("      \\addlinespace[2pt]
       \\hline \\\\[-1.8ex] \n\n")
 
@@ -375,7 +384,7 @@ print.aggTrees.inference <- function(x, table = "avg_char", ...) {
       \\hline \\\\[-1.8ex]
     \\end{tabular}
     \\end{adjustbox}
-    \\caption{Point estimates and $95\\%$ confidence intervals for the GATEs. Leaves are sorted in increasing order of the GATEs. Additionally, differences in the GATEs across all pairs of leaves are displayed. p-values to test the null hypothesis that a single difference is zero are adjusted using Holm's procedure and reported in parenthesis under each point estimate.}
+    \\caption{Point estimates and $95\\%$ confidence intervals for the GATEs based on asymptotic normality and on the percentiles of the bootstrap distribution. Leaves are sorted in increasing order of the GATEs. Additionally, differences in the GATEs across all pairs of leaves are displayed. p-values to test the null hypothesis that a single difference is zero are adjusted using Holm's procedure and reported in parenthesis under each point estimate.}
     \\label{table:differences.gates}
     \\end{table}
 \\endgroup \n\n")
